/**
 * Archivo que contiene brinda la funcionadad de monitorear los registros de EMA Libre.
 *
 * Contiene funcionen que controlan que se EMA se encuentra registrando correctamente
 * en caso contrario reiniciara el equipo (OJO: reinicia el sistema operativo).
 *
 * LICENSE:  This file is part of EMA Libre.
 * EMA Libre is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EMA Libre is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EMA Libre.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @copyright  Copyright (c) 2019 GUGLER (http://www.gugler.com.ar)
 * @license    https://www.gnu.org/licenses/gpl-3.0.html GPL License
 * @version    3.0
 * @link       http://www.gugler.com.ar
 * @since      File available since Release 3.0
 */

/**********************************************
        LIBRERIA PARA EL MONITOREO
***********************************************/
const fs = require('fs'); // Libreria fs para el manejo de archivos.
const C_EMA = require('./../conf/ConfiguracionGeneral.js'); // Configuracion General
const util = require('util'); // Utilidades.
const exec = require('child_process').exec; // Require child_process
const Correo = require('./mail.js'); // Envio de correo.

// FUNCION QUE CONTROLA EL ULTIMO REGISTRO

const umbral = 10; // tiempo maximo del ultimo registro en minutos antes de reiniciar.
const blucle = 3; // tiempo del blucle de control (cada cuantos minutos controla)
const bluclePrimerControl = 10; // tiempo del blucle del primer control

/*FUNCION AUXILIARES */  

function RegistrarLog (reg, depurar) {
    if (typeof depurar !== 'undefined') { depurar = false; }
    // se convierte la fecha a la UTC parametrizada
    var d = new Date();
    var utc = d.getTime() + (d.getTimezoneOffset() * 60000);
    var fechaHora = new Date(utc + (3600000 * (C_EMA.ConfGeneral.UTC)));
    var horas = fechaHora.getHours();
    var minutos = fechaHora.getMinutes();
    var segundos = fechaHora.getSeconds();
    var mes = fechaHora.getMonth() + 1;
    var dia = fechaHora.getDate();
    if (dia <= 9) { dia = '0' + dia; }
    if (mes <= 9) { mes = '0' + mes; }
    if (horas < 10) { horas = '0' + horas; }
    if (minutos < 10) { minutos = '0' + minutos; }
    if (segundos < 10) { segundos = '0' + segundos; }
    var fecha = dia + '/' + mes + '/' + fechaHora.getFullYear() + ' ' + horas + ':' + minutos + ':' + segundos;
    var registrologs = fecha + ' - ' + util.format(reg) + '\n';
    if (depurar) {
      // SE DEPURA EL ARCHIVO DE REGISTROS DE LOGS
      fs.writeFile('./logs/debug.log', '', { encoding: 'utf8', autoClose: true }, function (err) {
        if (err) { RegistrarLog('0|ERROR|Depurando archivo registro de logs :' + err); }
      });
    } else {
      // GUARDA LOS REGISTROS DE LOGS
      fs.appendFile('./logs/debug.log', registrologs, { encoding: 'utf8', autoClose: true }, function (err) {
        if (err) { RegistrarLog('0|ERROR|Guardando registro de logs en archivo :' + err); }
      });
    }
    registrologs=null;fecha=null;dia=null;mes=null;segundos=null;minutos=null;horas=null;fechaHora=null;utc=null;d=null;
  }


// Create shutdown function
function shutdown(callback){
    exec('/sbin/shutdown -r now', function(error, stdout, stderr){ callback(stdout); });
}



 /*FUNCION DE CONTROL */  
function controlUltimoRegistro () 
    {
     //Se lee el archivo de registro
        fs.readFile('./logs/registro_diario.log', {
            encoding: 'utf8',
            autoClose: true
        }, (err, data) => {
            var RegistroDiario = JSON.parse(data);
            var CantRegistros = RegistroDiario.length;
            //se obtiene el ultimo registro
            var fechaUltimo = RegistroDiario[CantRegistros-1].fecha;
            // Viene de esta forma: 07/10/2019 20:20:04
            var Dia = fechaUltimo.substring(0, 2);
            var Mes = fechaUltimo.substring(3, 5);
            var Anio = fechaUltimo.substring(6, 10);
            var Hora = fechaUltimo.substring(11, 13);
            var Min = fechaUltimo.substring(14, 16);
            var Segundos = fechaUltimo.substring(17, 19);           
            //se obtiene la fecha del ultmo registro como tipo Date
            var fechaUltDate = new Date(parseInt(Anio),(parseInt(Mes)-1),parseInt(Dia),parseInt(Hora),parseInt(Min),parseInt(Segundos)).getTime();
            var f = new Date();
            var FechaActual = new Date(f.getFullYear(), f.getMonth(),f.getDate(), f.getHours(),f.getMinutes(),f.getSeconds()).getTime();
            var diferenciaMinutos = 0;      
            diferenciaMinutos = ((FechaActual - fechaUltDate) / 1000)/60;
            // Se compara la diferencia con el umbral establecido en minutos
            if(diferenciaMinutos > umbral){
                // Supero el umbral se reinicia
                RegistrarLog("0|MONITOR|Supero el umbral. Tiempo: "+diferenciaMinutos.toFixed(2));
                if (C_EMA.ConfEmail.ActivarCorreos === true){
                    Correo.EnviarEmail(C_EMA,'EMA Libre - REINICIO AUTOMATICO', '<b>Estimado/a</b>,<br><br> Le informamos que la estación <b>' + C_EMA.ConfGeneral.NombreEstacion + '</b> dejó de recolectar datos por '+diferenciaMinutos.toFixed(2)+' minutos, por lo que se reinicio el equipo automáticamente.<br><br>Saludos Cordiales.');
                    }
                    // se reinicia el S.O (esperando 5 segundos para que se envie el correo si esta configurado)
                    setTimeout(function(){reiniciarSO();}, 5000);
            }else{
                // Dentro del umbral todo correcto
               RegistrarLog("0|MONITOR|Control dentro del umbral. Minutos: "+diferenciaMinutos.toFixed(2));
               setTimeout(function(){controlUltimoRegistro();}, (blucle*60*1000));
            }           
        });
      

    }

    
//Funcion de Reinicio del S.O

    function reiniciarSO () 
    {
        RegistrarLog("0|MONITOR|Se reinicio el S.O.");
        // Reinicia el equipo
       shutdown(function(output){ console.log(output);});
    }

//Se inicia el Monitor luego tiempo definido en el umbral.
setTimeout(function() { RegistrarLog("0|MONITOR|Se inicio el monitor de EMA Libre.");
                        RegistrarLog("0|MONITOR|Tiempo de control cada "+blucle+" minutos.");
                        controlUltimoRegistro(); }, (bluclePrimerControl*60*1000));

