/**
 * Este archivo es parte de EMA Libre y brinda la funcionalidad de envio de datos
 * metorologicos recolectados por EMA Libre a una red WINDY (https://stations.windy.com/)
 * Este archivo, contiene metodos que se encargan de enviar (por POST o GET) y detectar si dicho envio
 * fue recibido correctamente por la red, basandose en el mensaje emitido por la red.
 *
 * LIMITE: Envio de registros con frecuencia mayor o igual a 5 minutos.
 * 
 * LICENSE:  This file is part of EMA Libre.
 * EMA Libre is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EMA Libre is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EMA Libre.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @copyright  Copyright (c) 2017 GUGLER (http://www.gugler.com.ar)
 * @license    https://www.gnu.org/licenses/gpl-3.0.html GPL License
 * @version    3.0
 * @link       http://www.gugler.com.ar
 * @since      File available since Release 3.0
 */


var http = require('https');
var FechaUltimoPublicadoWindy = "S/D";

(function () {
  'use strict';

  var fields = [
    'ID',
    'station',
    'time',
    'dateutc',
    'ts',
    'temp',
    'tempf',
    'wind',
    'windspeedmph',
    'winddir',
    'gust',
    'windgustmph',
    'rh',
    'dewpoint',
    'pressure',
    'mbar',
    'baromin',
    'precip',
    'rainin',
    'uv'
  ];

  var timeout = 5000;

  function WINDY(ID, PASSWORD) {


    var server = {
      host: 'stations.windy.com',
      port: 443,
      fijo: '/pws/update/'+ID,  // Se agrega la KEY de la estacion
      path: '/pws/update/'+ID, // Se agrega la KEY de la estacion
      method: 'GET'
    };

    if (!(this instanceof WINDY)) return new WINDY(ID, PASSWORD);

    var payload = {
      station: PASSWORD || ''
    };

    this.setObservations = function () {

      if (arguments.length > 0) {
        switch (typeof arguments[0]) {
          case 'object':
            var observations = arguments[0];
            if (Object.prototype.toString.call(observations) === '[object Object]') {
              Object.keys(observations).forEach(function (observation) {
                if (fields.indexOf(observation) > -1) {
                  payload[observation] = String(observations[observation]).toString();
                }
              });
              return true;
            } else {
              return new Error('Invalid argument for setObservations().');
            }
            break;

          case 'string':
            var observation = arguments[0];
            var reading = arguments[1] || '';
            if (fields.indexOf(observation) > -1) {
              payload[observation] = reading;
              return true;
            } else {
              return new Error('Observation ' + observation + ' not supported by WU.');
            }
            break;

          default:
            return new Error('Invalid argument for setObservations().');
        }
      } else {
        return new Error('No argument supplied to setObservations().');
      }
    };

    this.getObservations = function () {
      return payload;
    };

    this.resetObservations = function () {
      payload = {
        station: payload.station,
        dateutc: 'now'
      };
      return true;
    };

    this.getFields = function () {
      return fields;
    };

    this.setRequestTimeout = function (miliseconds) {
      if (!isNaN(Number(miliseconds))) {
        timeout = miliseconds;
        return timeout;
      } else {
        return new Error('Invalid timeout.');
      }
    };

    this.getRequestTimeout = function () {
      return timeout;
    };

    this.sendObservations = function (callback) {
  
      if (typeof callback !== 'function') callback = function () {};
      var queryString = [];
      var that = this;
      // SE CONTROLA QUE EL REGISTRO SEA MAYOR A 5 QUE EL ANTERIOR
      //se obtiene el ultimo registro
      if(FechaUltimoPublicadoWindy != "S/D")
      {
        // Viene de esta forma: 07/10/2019 20:20:04
        var DiaT = FechaUltimoPublicadoWindy.substring(0, 2);
        var MesT = FechaUltimoPublicadoWindy.substring(3, 5);
        var AnioT = FechaUltimoPublicadoWindy.substring(6, 10);
        var HoraT = FechaUltimoPublicadoWindy.substring(11, 13);
        var MinT = FechaUltimoPublicadoWindy.substring(14, 16);
        var SegundosT = FechaUltimoPublicadoWindy.substring(17, 19);           
        //se obtiene la fecha del ultmo registro como tipo Date
        var fechaUltDate = new Date(parseInt(AnioT),(parseInt(MesT)-1),parseInt(DiaT),parseInt(HoraT),parseInt(MinT),parseInt(SegundosT)).getTime();
        var f = new Date();
        var FechaActual = new Date(f.getFullYear(), f.getMonth(),f.getDate(), f.getHours(),f.getMinutes(),f.getSeconds()).getTime();
        var diferenciaMinutos = 0;      
        diferenciaMinutos = ((FechaActual - fechaUltDate) / 1000)/60;
      }
      if(diferenciaMinutos < 5.5 && FechaUltimoPublicadoWindy != "S/D")
          {  
            that.resetObservations(); callback(null, [430, FechaUltimoPublicadoWindy]);
          }
          else{
            var d = new Date();
            var utc = d.getTime() + (d.getTimezoneOffset() * 60000);
            var db = new Date(utc + (3600000 * (APP.Configuracion.utc)));
            var hora = db.getHours();
            if (hora <= 9) { hora = '0' + hora };
            var mes = db.getMonth() + 1;
            if (mes <= 9) { mes = '0' + mes };
            var dia = db.getDate();
            if (dia <= 9) { dia = '0' + dia };
            var minutos = db.getMinutes();
            if (minutos <= 9) { minutos = '0' + minutos };
            var hora = db.getHours();
            if (hora <= 9) { hora = '0' + hora };
            var segundo = db.getSeconds();
            if (segundo <= 9) { segundo = '0' + segundo };
            var fecha = dia + "/" + mes + "/" + db.getFullYear() + " " + hora + ':' + minutos + ':' + segundo;            
              Object.keys(payload).forEach(function (observation) {
                queryString.push(observation + '=' + encodeURIComponent(payload[observation]).replace('%2B', '+'));
              });
              server.path = server.fijo;
              server.path += '?' + queryString.join('&');
              var request = http.request(server, function (response) {
                var message = '';
                response.on('data', function (chunk) {
                  message += chunk;
                });

                response.on('end', function () {
                  var result = message.trim();
                  if (result === 'SUCCESS') {
                    // Registro publicado correctamente
                    FechaUltimoPublicadoWindy = fecha;
                    that.resetObservations();
                    callback(null, result);
                  } else {
                    
                          if (result.substr(result, 8) == 'SUCCESS ' )
                          { // Esta Congestionada
                            that.resetObservations();
                            callback(null, [429, FechaUltimoPublicadoWindy]);
                          } else {
                                // Otro error.
                                var detalle_error = result.substr(result, 100);
                                callback(new Error(detalle_error), null);
                                }
                        }
                });
              });

              request.on('socket', function (socket) {
                socket.setTimeout(timeout);
                socket.on('timeout', function () {
                  request.abort();
                });
              }).on('error', function (error) {
                callback(error, null);
              });
              request.end();
          }
      };
  }
  module.exports = WINDY;
}());
