/**
 * Este archivo es parte del proyecto EMA Libre.
 *
 * Contiene el codigo fuente, cuya funcion principal
 * es presentar una interfaz interactiva en modo consola o shell
 * permitiendo iniciar, detener y visualizar el estado y los registros 
 * de EMA Libre Cliente, tanto en modo web como en modo consola nativo.
 *
 * LICENSE:  This file is part of EMA Libre.
 * EMA Libre is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EMA Libre is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EMA Libre.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @copyright  Copyright (c) 2019 GUGLER (http://www.gugler.com.ar)
 * @license    https://www.gnu.org/licenses/gpl-3.0.html GPL License
 * @version    3.0
 * @link       http://www.gugler.com.ar
 * @since      File available since Release 3.0
 */

// Require the lib, get a working terminal
//const request = require('request');
const http = require('http');
const term = require( 'terminal-kit' ).terminal ;
const fsconf = require('fs');
var configGeneralEMA =  eval(fsconf.readFileSync('./conf/ConfiguracionGeneral.js')+'');
var configGlobalEMA =  eval(fsconf.readFileSync('./conf/ConfiguracionGlobal.js')+'');
var intervalo; 
var modoTemp;
// CODIGO PRINCIPAL

function Principal(continuo)
{   
    consultarModo().then(modo =>
        {
         if(continuo == false) {ArmarMenu(modo);}
         if(!(JSON.stringify(modo) === JSON.stringify(modoTemp))) {ArmarMenu(modo);}
         MostrarUltimoRegistro(modo);
        });
}

/////////////////////////
// METODOS Y FUNCIONES //
/////////////////////////

function Salir(){
    clearInterval(intervalo);
    process.exit() ;
}

//Consulta si existe el modo Carpincho Web
const consultarModo =  function (){
                                    return new Promise((resultado, error)=>
                                        {
                                        let modoWeb= false;
                                        let estadoServicio = false;
                                        let archivopid; 
                                        let pidNodeConsola = false;  
                                        let estadoweb= false 
                                       // Se encuentra corriendo node consola
                                        if (fsconf.existsSync("/var/run/emalibre.pid"))
                                             {
                                              archivopid = fsconf.readFileSync('/var/run/emalibre.pid');
                                              //archivopid = JSON.parse(archivopid);
                                              pidNodeConsola = archivopid;
                                              estadoConsola = true;
                                              }else{estadoConsola = false;}
                                        // Existe el modo web instalado
                                        if (fsconf.existsSync("./gui_web"))
                                        {
                                            modoWeb= true;
                                            http.get('http://127.0.0.1:8080/EMA_Estado', (res) => {
                                                const { statusCode } = res;
                                                if (statusCode == 200)
                                                { // Web corriendo
                                                  estadoServicio = true;
                                                }else{ // Web detenido
                                                    estadoServicio = false;
                                                     }
                                                res.setEncoding('utf8');
                                                let rawData = '';
                                                res.on('data', (chunk) => { rawData += chunk; });
                                                res.on('end', () => {
                                                try {
                                                    const parsedData = JSON.parse(rawData);
                                                    estadoServicio = true;
                                                    estadoweb = parsedData[0];
                                                    resultado([modoWeb,estadoServicio, estadoweb, estadoConsola, pidNodeConsola]);
                                                } catch (e) {
                                                    estadoServicio = false;
                                                    estadoweb = false;
                                                    resultado([modoWeb,estadoServicio, estadoweb, estadoConsola, pidNodeConsola]);
                                                }       
                                                  }); 
                                            });
                                        }else{
                                            modoWeb=false;estadoServicio=false;estadoweb=false;
                                            resultado([modoWeb,estadoServicio, estadoweb, estadoConsola, pidNodeConsola]);
                                            }
                                        })
                                    }

function controlarTamañoConsola()
    {
    if(term.width < 68 || term.height < 25)
        {
        term.reset();
        term.wrap.red('\n\nImposible abir la consola, con el tamañno actual de la terminal.');  
        term.wrap.red('\nTamaño Actual->Ancho: '+term.width+' Alto:'+term.height)
        term.wrap.cyan('\nConfigurar al menos 25x68\n\n\n');
        Salir();
        }
    }

function ArmarMenu (modo)
    {
     modoTemp = modo;
    //EL MODO TIENE true/false de la siguiente forma: [web,estoservicio, estadoweb, estadoconsola]
    term.reset();    
    var options = {
        y: 1 ,
        separator: '   ',
        style: term.inverse ,
        selectedStyle: term.dim.blue.bgGreen
    } ;
    var items;
    let texto= fsconf.readFileSync('/etc/emalibre_modo');
    items = [ 'EMA Libre                                  ' , 'Logs       ' , 'Salir' ] ;
    term.singleLineMenu( items , options , function( error , response ) {
                       term( '\n' ).eraseLineAfter.green( Menu(response.selectedText)) ;      
                       });
   
    let modotexto;
    if(modo[0]){modotexto ="Web/Consola";}else{modotexto="Consola";}
    term.moveTo( 2 , 2 , '' ) ;
    term.wrapColumn( { x: 2 , width: 50, height: 2, offset: 0, continue:true } );
    term.wrap.brightCyan('Modos: ');
    term.wrap.white(modotexto); 
    term.wrap.brightCyan('  Modo Actual: '); 
    term.wrap.brightGreen(texto); 
    }

function MostrarUltimoRegistro(modo)
    {
      obtenerTanques();
    try{
      term.moveTo( 2 , 3 , '' ) ;
      term.wrapColumn( { x: 1 , width: 50, height: 50, offset: 0, continue:true } );
      term.wrap.brightCyan('Version: '); 
      term.wrap.brightWhite(configGlobalEMA.ConfGlobal.Version + 'v');
      term.wrap.brightCyan('      Recolecta: '); 
      term.wrap.brightGreen(configGeneralEMA.ConfGeneral.Tiempo_de_Recoleccion + ' s');
      term.wrap.brightCyan('   Modo Operación: '); 
      term.wrap.brightGreen(configGeneralEMA.ConfGeneral.Modo);   
      term.wrap.cyan('\n' );      
     if(modo[0] && modo[1])
       {  
        // Se consulta por API (modo web) 
        // ESTADO
        http.get('http://127.0.0.1:8080/EMA_Estado', (res) => {
            const { statusCode } = res;
            term.moveTo( 2 , 4 , '' ) ;
            if (statusCode == 200)
            { // Web corriendo
              estadoServicio = true;
            }else{ // Web detenido
                estadoServicio = false;
                 }
            res.setEncoding('utf8');
            let rawData = '';
            res.on('data', (chunk) => { rawData += chunk; });
            res.on('end', () => {
            try {
                term.wrapColumn( { x: 1 , width: 50, height: 50, offset: 0, continue:true } );
                const parsedData = JSON.parse(rawData);
                estadoweb = parsedData[0];
                var estado;
                term.wrap.brightCyan('Estado: ' );
                if(estadoweb == true){estado='Encendida';term.wrap.brightGreen(estado); }
                else{estado='Detenida ';term.wrap.brightRed(estado);}      

            } catch (e) {
                term.reset(); 
                term.wrap.brightGreen('\nNo esta corriendo el servicio web\n');
                Principal(false);
                return true;
            }       
              }); 
        });

        //Ultimo Registro
        http.get('http://127.0.0.1:8080/UltimoRegistro', (res) => {
            const { statusCode } = res;
            term.moveTo( 2 , 4 , '' ) ;
            res.setEncoding('utf8');
            let rawData = '';
            res.on('data', (chunk) => { rawData += chunk; });
            res.on('end', () => {
                    try {
                        body  = JSON.parse(rawData);
                        term.moveTo( 22 , 4 , '' ) ;     
                        term.wrapColumn( { x: 2 , width: 31, height: 9, offset: 0, continue:true } );
                        term.wrap.brightCyan('Ult. Regi: ' );  
                        term.wrap.brightGreen(body.fecha);
                        term.moveTo( 2 , 6 , '' ) ;            
                        term.wrapColumn( { x: 2 , width: 24, height: 9, offset: 0, continue:true } );
                        term.wrap.yellow('DATOS EXTERNOS' );
                        term.wrap.cyan('\nTemperatura: ' );
                        term.wrap.brightWhite((body.temperatura_externa || 'S/D') +'°C   ');
                        term.wrap.cyan('\nHumedad: ' );
                        term.wrap.brightWhite((body.humedad_externa || 'S/D') + '%   ');
                        term.wrap.cyan('\nPresion R: ' );
                        term.wrap.brightWhite((body.presion_relativa || 'S/D') + ' hpa   ');
                        term.wrap.cyan('\nPunto de rocio: ' );
                        term.wrap.brightWhite((body.punto_de_rocio || 'S/D') + '°C   ');
                        term.wrap.cyan('\nIluminancia: ' );
                        term.wrap.brightWhite((body.luxer_intencidad || '0') + ' lux   ');
                        term.wrap.cyan('\nUV: ' );     
                        term.wrap.brightWhite((body.luxer_uv || '0') + ' uv   ');
                        term.wrap.cyan('\nViento V.: ' );     
                        term.wrap.brightWhite((body.viento_velocidad || 'S/D') + ' Km/h   ');
                        term.wrap.cyan('\nViento R.: ' );     
                        term.wrap.brightWhite((body.viento_rafagas || 'S/D') + ' Km/h   ');
                        term.wrap.cyan('\nViento N.: ' );     
                        term.wrap.brightWhite((body.viento_direccion_nombre || 'S/D')+ '     ');
                        term.wrap.yellow('\n\nLLUVIA ACUMULADA' );
                        term.wrap.cyan('\nActual: ') 
                        term.wrap.brightWhite((body.lloviendo || '0') + ' mm  ');     
                        term.wrap.cyan(' Hora: ') 
                        term.wrap.brightWhite((body.lluvia_acu_hora || '0') + ' mm  ');
                        term.wrap.cyan(' Diaria: ' );     
                        term.wrap.brightWhite((body.lluvia_acu_diaria || '0') + ' mm  ');
                        term.wrap.cyan(' Total: ' );     
                        term.wrap.brightWhite((body.lluvia_acumulado_total || '0') + ' mm  ');                         
                        term.moveTo( 28 , 6 , '' ) ;
                        term.wrapColumn( { x: 28 , width: 24, height: 3, offset: 0, continue:true } );
                        term.wrap.yellow('DATOS INTERNOS' );
                        term.wrap.cyan('\nTemperatura: ' );     
                        term.wrap.brightWhite((body.temperatura_interna || 'S/D') +'°C   '); 
                        term.wrap.cyan('\nHumedad: ' );
                        term.wrap.brightWhite((body.humedad_interna || 'S/D') + '%   ');   

                    } catch (e) {
                        term.moveTo( 2 , 6 , '' ) ; 
                        term.wrap.yellow('SIN DATOS' );
                        return false;
                    }
              }); 
        });
       //Ultimo Registro REDES ENVIADAS
        http.get('http://127.0.0.1:8080/UltimoRegistro', (res) => {
            const { statusCode } = res;
            term.moveTo( 1 , 19 , '' ) ;
            term.wrapColumn( { x: 2 , width: 80, height: 50, offset: 0, continue:true } );            
            term.wrap.yellow('\nREDES ENVIADAS' );
            if (statusCode == 200)
            { // Web corriendo
              estadoServicio = true;
            }else{ // Web detenido
                estadoServicio = false;
                 }
            res.setEncoding('utf8');
            let rawData = '';
            res.on('data', (chunk) => { rawData += chunk; });
            res.on('end', () => {
                    try {
                        body  = JSON.parse(rawData);
                        if(typeof body[0] !== 'undefined' && body[0] !== null)
                        {
                            for (var i=0; i < body.length - 1; ++i)
                            { 
                                if(typeof body[i].red !== 'undefined' && body[i].red !== null)
                                {              
                                term.wrap.cyan('\n'+(body[i].red || 0)+':' );     
                                if (body[i].estado){
                                    term.wrap.white(body[i].fecha || 0);
                                    term.wrap.green(' - '+ (body[i].estado || false) +'           ');
                                    }else{term.wrap.brightRed(body[i].fecha || false);
                                        term.wrap.red(' - '+ (body[i].estado || false) +'         ');
                                        }
                                }
                            }
                        }else{  term.wrap.cyan('\n S/D'); }                     
                        term.moveTo( 1 , 1 , '' ) ; 

                    } catch (e) {
                                return false;
                    }
              }); 
        }); 


        }else
        { // Modo Consola consultar por archivo
            obtenerRegistroConsola();
        }     
    } catch(err){
        Principal(false);
        }                       
    // Al final se posiciona en el punto inicial
    term.moveTo( 1 , 1 , '' ) ; 
    }


function obtenerRegistroConsola ()
    {    
        try{
        //Se trae el archivo de registros diarios y se parcea.
        let Archivo = fsconf.readFileSync('./logs/registro_diario.log', {encoding: 'utf8',  autoClose: true});
        let RegistroDiario = JSON.parse(Archivo);
        let UltimoRegistro = RegistroDiario.length - 1;
        if(UltimoRegistro<1){   UltimoRegistro = 0;
                                RegistroDiarioT = {
                                                fecha: '-',
                                                temperatura_externa: '-',
                                                humedad_externa: '-',
                                                punto_de_rocio: '-',
                                                luxer_intencidad: '-',
                                                luxer_uv: '-',
                                                viento_velocidad: '-',
                                                presion_relativa: '-',
                                                viento_rafagas: '-',
                                                viento_direccion_nombre: '-',
                                                lloviendo: '-',
                                                lluvia_acu_hora: '-',
                                                lluvia_acu_diaria: '-',
                                                lluvia_acumulado_total: '-',
                                                temperatura_interna: '-',
                                                humedad_interna: '-'
                                                };
                             RegistroDiario.push(RegistroDiarioT);
                           }
        //ESTADO
        term.moveTo( 2 , 4 , '' ) ;
        term.wrapColumn( { x: 1 , width: 50, height: 50, offset: 0, continue:true } );
        let estado;
        term.wrap.brightCyan('Estado: ' );
        if(fsconf.existsSync("/var/run/emalibre.pid")){estado='Encendida';term.wrap.brightGreen(estado); }
                        else{estado='Detenida ';term.wrap.brightRed(estado);}    
        // ULTIMO REGISTRO
        term.moveTo( 28 , 4 , '' ) ;     
        term.wrapColumn( { x: 2 , width: 31, height: 9, offset: 0, continue:true } );
        term.wrap.brightCyan('Ult. Regi: ' );  
        term.wrap.yellow( RegistroDiario[UltimoRegistro].fecha);
        term.moveTo( 2 , 6 , '' ) ;    
        term.wrapColumn( { x: 2 , width: 24, height: 9, offset: 0, continue:true } );
        term.wrap.yellow('DATOS EXTERNOS' );
        term.wrap.cyan('\nTemperatura: ' );
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].temperatura_externa || 'S/D') +'°C   ');
        term.wrap.cyan('\nHumedad: ' );
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].humedad_externa || 'S/D') + '%   ');
        term.wrap.cyan('\nPresion R: ' );
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].presion_relativa || 'S/D') + ' hpa   ');
        term.wrap.cyan('\nPunto de rocio: ' );
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].punto_de_rocio || 'S/D') + '°C   ');
        term.wrap.cyan('\nIluminancia: ' );
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].luxer_intencidad || '0') + ' lux   ');
        term.wrap.cyan('\nUV: ' );     
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].luxer_uv || '0') + ' uv   ');
        term.wrap.cyan('\nViento V.: ' );     
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].viento_velocidad || 'S/D') + ' Km/h   ');
        term.wrap.cyan('\nViento R.: ' );     
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].viento_rafagas || 'S/D') + ' Km/h   ');
        term.wrap.cyan('\nViento N.: ' );     
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].viento_direccion_nombre || 'S/D')+ '     ');
        term.wrap.yellow('\n\nLLUVIA ACUMULADA' );
        term.wrap.cyan('\nActual: ') 
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].lloviendo || '0') + ' mm  ');     
        term.wrap.cyan(' Hora: ') 
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].lluvia_acu_hora || '0') + ' mm  ');
        term.wrap.cyan(' Diaria: ' );     
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].lluvia_acu_diaria || '0') + ' mm  ');
        term.wrap.cyan(' Total: ' );     
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].lluvia_acumulado_total || '0') + ' mm  ');                         
        term.moveTo( 28 , 6 , '' ) ;
        term.wrapColumn( { x: 28 , width: 24, height: 3, offset: 0, continue:true } );
        term.wrap.yellow('DATOS INTERNOS' );
        term.wrap.cyan('\nTemperatura: ' );     
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].temperatura_interna || 'S/D') +'°C   '); 
        term.wrap.cyan('\nHumedad: ' );
        term.wrap.brightWhite((RegistroDiario[UltimoRegistro].humedad_interna || 'S/D') + '%   ');
     } catch(err){
                  setTimeout(function () { Principal(false);}, 2000);
                  }

    }    
function obtenerTanques ()
    {    
    // Aplica en todos los modos (mira archivos)
        //Datos de los archivo de LOGS, REGISTROS y TANQUE
        term.moveTo( 11 , 9 , '' ) ;
        term.wrapColumn( { x: 28 , width: 50, height: 50, offset: 0, continue:true } );
        // TANQUE DIARIO
        term.wrap.yellow('\nTANQUES DEL SISTEMA' );
        if (fsconf.existsSync("./logs/registro_diario.log"))
        {let estadoarchivoTD = fsconf.statSync("./logs/registro_diario.log");
            let TamanioenMegaTD = (estadoarchivoTD.size)/ 1000000.0;
            term.wrap.cyan('\nTanque Diario: ' );     
            term.wrap.brightWhite(TamanioenMegaTD.toFixed(2) + ' Mb');  
            estadoarchivoTD=null;TamanioenMegaTD=null;
        }else{term.wrap.brightWhite('No existe');}
        // TANQUE PENDIENTE
        if (fsconf.existsSync("./logs/TanquePendientes.log"))
            {var estadoarchivoTP = fsconf.statSync("./logs/TanquePendientes.log");
                var TamanioenMegaTP = (estadoarchivoTP.size)/ 1000000.0;
                term.wrap.cyan('\nTanque Pendiente: ' );     
                term.wrap.brightWhite(TamanioenMegaTP.toFixed(2) + ' Mb');  
                estadoarchivoTP=null;TamanioenMegaTP=null;
            }else{term.wrap.brightWhite('No existe');}
        // ARHCIVO DE LOGS
        if (fsconf.existsSync("./logs/debug.log"))
            {var estadoarchivoD = fsconf.statSync("./logs/debug.log");
                var TamanioenMegaD = (estadoarchivoD.size)/ 1000000.0;
                term.wrap.cyan('\nTanque Debug: ' );     
                term.wrap.brightWhite(TamanioenMegaD.toFixed(2) + ' Mb'); 
                estadoarchivoD=null;TamanioenMegaD=null;
            }else{term.wrap.brightWhite('No existe');}
        term.moveTo( 1 , 1 , '' ) ; 
        }            
function obtenerLog ()
    {
    term.moveTo( 1 , 2 , '' ) ;
    term.wrapColumn( { x: 0 , width: 80, height: 50, offset: 0, continue:true } );
    term.wrap.cyan('REGISTROS EN TIEMPO REAL                                 ');
    term.moveTo( 1 , 3 , '' ) ;
    term.wrapColumn( { x: 0 , width: 80, height: 50, offset: 0, continue:true } );
    let Archivo = fsconf.readFileSync('./logs/debug.log', {encoding: 'utf8',  autoClose: true});
    Archivo = Archivo.split('\n');
    let CantidadRegistrosInicio = Archivo.length - 22
    if (CantidadRegistrosInicio < 10) {        
        term.wrap.cyan('FECHA       HORA    | DETALLE DEL REGISTRO');
        term.wrap.cyan('\n' );
        term.wrap.cyan('  --------- NO HAY REGISTROS -------------');
        for (var i=0; i < 19; ++i)
        {term.brightWhite("\n                                                           ");}        
      } else {     
                term.wrap.cyan('FECHA       HORA    | DETALLE DEL REGISTRO');
                term.wrap.cyan('\n' );          
                for (var i=CantidadRegistrosInicio; i < Archivo.length - 1; ++i)
                {
                let Resultado = Archivo[i].split("|");
                term.wrap.eraseLineAfter.brightWhite(Resultado[0].substring(0,16) +'|' + Resultado[2].substring(0,48)); 
                term.wrap.eraseLineAfter.cyan('\n' );
                }  
                term.moveTo( 1 , 25 , '' ) ;
                term.wrap.cyan('---------------------------------------------------------');
                term.moveTo( 1 , 1 , '' ) ; 
            }
   }

function Menu(elemento){
    clearInterval(intervalo);
    intervalo = setInterval(function () { Principal(true);}, 2000);
    switch (elemento) {
        case 'EMA Libre                                  ':
            // Nada solo se refresca 
            break;                           
        case 'Logs       ':
            clearInterval(intervalo);
            intervalo = setInterval(function () { obtenerLog();}, 3000);
            term.moveTo( 1 , 2 , '' ) ; 
            term.brightWhite( "\n-----------------------------------------------------------");            
            term.brightWhite( "\nObteniendo los logs del sistema, Espere               ...  ");
            term.brightWhite( "\n-----------------------------------------------------------");
            for (var i=0; i < 19; ++i)
            {term.brightWhite("\n                                                           ");}
            break;
        case 'Salir':
            term.reset();
            term.white( "Hasta Luego!!!") ;
            term.magenta('\n' ); 
            Salir();                      
            break;                           
        default:
            term.magenta( "No existe menu creado para: "+ elemento ) ;
        }
setTimeout(function () { Principal(false);}, 2000);
}

// Intervalo Continuo 
term.reset(); 
Principal(false);
intervalo = setInterval(function () { Principal(true);}, 2000);
 