/**
 * Este archivo es parte del proyecto EMA Libre.
 * 
 * El codigo fuente, brinda la funcionalidad de validacion basica de los
 * de datos metorologicos recolectados.
 *
 * Este archivo, contiene metodos que validan si cada registro recolectado(temperatura, humedad, etc)
 * esta dentro de los rangos estipulados como correctos, en caso contrario devuelve un error
 * indicando que validacion no fue superada.
 *
 * LICENSE:  This file is part of EMA Libre.
 * EMA Libre is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EMA Libre is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EMA Libre.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @copyright  Copyright (c) 2019 GUGLER (http://www.gugler.com.ar)
 * @license    https://www.gnu.org/licenses/gpl-3.0.html GPL License
 * @version    3.0
 * @link       http://www.gugler.com.ar
 * @since      File available since Release 3.0
 */

module.exports = {
  Registros: function (Parametros, callback) {
    switch (Parametros[0]) {
      case 'validar':
        var Bandera = 0;
        var textoerror = '';

        // Se valida los datos recolectados

        // Sensores Externos
        if (Parametros[1][17].rain_overflow === true) {
          textoerror += ' VAL: Problemas con el sensor de lluvia.';
          Bandera += 1;
        }
        if (Parametros[1][17].lost_sensor_contact === true) {
          textoerror += ' VAL: Problemas con la RF de la estacion externa.';
          Bandera += 1;
        }
        // temperatura externa
        if (Parametros[1][2] < -40 || Parametros[1][2] > 60 || Parametros[1][2] == null) {
          textoerror += ' VAL: Temp. Externa: ' + Parametros[1][2];
          Bandera += 1;
        }
        // temperatura interna
        if (Parametros[1][3] < -10 || Parametros[1][3] > 60 || Parametros[1][3] == null) {
          textoerror += ' VAL: Temp. Interna: ' + Parametros[1][3];
          Bandera += 1;
        }
        //  Humedad Interna
        if ( Parametros[1][4] < 10 || Parametros[1][4] > 99 ||Parametros[1][4] == null) {
          Bandera += 1;
          textoerror += ' VAL: Humedad Interna: ' + Parametros[1][4];
        }
        //  Humedad Externa
        if (Parametros[1][5] < 10 || Parametros[1][5] > 99  || Parametros[1][5] == null) {
          textoerror += ' VAL: Humedad Externa: ' + Parametros[1][5];
          Bandera += 1;
        }
        //  Luz Intencidad (luxer) - No obligatoria
        if (Parametros[1][6] < 0 || Parametros[1][6] > 400000 ||Parametros[1][6] == null) {
          textoerror += ' VAL(no obligatoria): Luz Intencidad: ' + Parametros[1][6];
        }
        //  Rayos UV  - No obligatoria
        if (Parametros[1][7] > 20 || Parametros[1][7] < 0 || Parametros[1][7] == null) {
          textoerror += ' VAL(no obligatoria): Rayos UV: ' + Parametros[1][7];
        }
        //  Viento  vel.
        if (Parametros[1][8] < 0 || Parametros[1][8] > 180 || Parametros[1][8] == null) {
          Bandera += 1;
          textoerror += ' VAL: Viento vel.: ' + Parametros[1][8];
        }
        //  Viento Rafagas
        if (Parametros[1][9] < 0 || Parametros[1][9] > 180 || Parametros[1][9] == null) {
          Bandera += 1;
          textoerror += ' VAL: Viento rafagas: ' + Parametros[1][9];
        }
        //  Viento Direccion
        if (Parametros[1][10] < 0 || Parametros[1][10] > 360 || Parametros[1][10] == null) {
          Bandera += 1;
          textoerror += ' VAL: Viento Dir. fuera de rango: ' + Parametros[1][10];
        }
        //  P. Absoluta
        if (Parametros[1][12] < 0 || Parametros[1][12] > 1100 || Parametros[1][12] == null) {
          textoerror += ' VAL: Presion Absoluta: ' + Parametros[1][12];
          Bandera += 1;
        }
        //  P.Relativa
        if (Parametros[1][13] < 500 || Parametros[1][13] > 1100 ||Parametros[1][13] == null) {
          textoerror += ' VAL: Presion Relativa: ' + Parametros[1][13];
          Bandera += 1;
        }
        //  P.Lluvia Totoal
        if (Parametros[1][14] < 0 || Parametros[1][14] > 999999 ||Parametros[1][14] == null) {
          textoerror += ' VAL: Lluvia total: ' + Parametros[1][14];
          Bandera += 1;
        }
        //  Punto Rocio
        if (Parametros[1][15] > 100 || Parametros[1][15] < -50 || Parametros[1][15] == null) {
          textoerror += ' VAL: Punto Rocio: ' + Parametros[1][15];
          Bandera += 1;
        }
        if (Bandera === 0) {
          callback(null, true);
        } else {
          callback([Bandera,textoerror], false);
        }
        Bandera = null;
        textoerror = null;
        break;
      default:
        console.log('N/A Ningun Tipo de validacion');
    }
  }
};
