/**
 * Archivo principal de la administracion grafica de la aplicacion EMA Libre.
 *
 * Archivo de inicio el entorno grafico GUI del sistema EMA Libre que nos
 * brinda una pantalla con las opciones: Configuracion general del sistema,
 * ABM de Redes Meteorologicas,Visualizacion del sensores e informacion enviadas a la redes y
 * reportes de log, como historicos registrados.
 *
 * LICENSE:  This file is part of EMA Libre.
 * EMA Libre is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EMA Libre is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EMA Libre.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @copyright  Copyright (c) 2019 GUGLER (http://www.gugler.com.ar)
 * @license    https://www.gnu.org/licenses/gpl-3.0.html GPL License
 * @version    3.0
 * @link       http://www.gugler.com.ar
 * @since      File available since Release 3.0
 */

// **********************Dependencias **************************
//const configGeneralEMA = require('./../conf/ConfiguracionGeneral.js'); // Configuracion General
//const configGlobalEMA = require('./../conf/ConfiguracionGlobal.js'); // Configuracion Global
//const funcionesActualizar = require('./../core/actualizar.js'); // Se incluye el core de EMA.
//const EMA = require('./../core/ema.js'); // Se incluye el core de EMA.
//const gui = require('nw.gui'); // Libreria Grafica
//gui.App.clearCache();     // Se limpia cache
//win = gui.Window.get(); // Se obtiene la ventana actual
var configGeneralEMA;
var configGlobalEMA;
$.get("/configGeneralEMA", function(data, status){configGeneralEMA = data;});
$.get("/configGlobalEMA", function(data, status){configGlobalEMA = data; });

// **********************Vairales Generales*********************
const iconoEncendido = '/iconos/icono_enc.png'; // Imagen del Icono Encendido
const iconoApagado = '/iconos/icono_off.png'; // Imagen del Icono Apagado
const iconoPublicado = '/iconos/icono_pub.png'; // Imagen del Icono Apagado
const iconoNeutro = '/img/neutro.webp'; // Imagen del Icono Neutro
const iconoControlTrue = '/img/ok.webp'; // Imagen de Control TRUE
const iconoControlFalse = '/img/no.webp'; // Imagen de Control FALSE
var UltimoReg;
var Ant_UltimoReg;
var ModeloVisor; // Variables Generales - Registros recolectados
var destanqueo = 0;
var intervaloBarra = 0; 
var cant = 0;
var tiempovisor=99;

// **********************Funciones Principales de la GUI *****************************************************

const esperar = ms => new Promise(function(resolve){setTimeout(resolve, ms);});

// Captura el redimencionamiento de la ventana

 $(window).on('resize', function () {
  var zoomLevel=0;
  var width = $(window).width();
  var height = $(window).height();
  if (width < 300 && height < 300) { zoomLevel = 0.6; }
  if (width > 300 && height > 300) { zoomLevel = 0.5; }
  if (width > 350 && height > 350) { zoomLevel = 0.4; }
  if (width > 800 && height > 600) { zoomLevel = 0; }
  if (width > 850 && height > 690) { zoomLevel = 0; }
  if (width > 1000 && height > 800) { zoomLevel = 0; }
  if (width > 1200 && height > 820) { zoomLevel = 1.2; }
  if (width > 1200 && height > 900) { zoomLevel = 1.3; }
  if (width > 1400 && height > 1000) { zoomLevel = 1.4; }
 // console.log(zoomLevel);
  $('html').css({zoom: zoomLevel});
});


function ComprobarEstado () {

  $.get("/EMA_estado", function(data, status){
  var EstadoEMA_Control = data; 
  if(EstadoEMA_Control[0])
    {  
    HABLA(0, 'Ya me encuentro recolectando ..', 'success');
    $('#signal').addClass('parpadea');    
    $('td#barralateral').css('background-color', '#11881161');
    $('#estado').html('En Linea');
    $('#mensaje').html('Encendido');
    $('#botoniniciar').attr('disabled', true);
    $('#botondetener').attr('disabled', false);
    RegistroActual();
    clearInterval(intervaloBarra);
    intervaloBarra = window.setInterval(function () { BarradeProgreso();}, 200);
    }
  });
}

function INICIAR () {
  if (configGeneralEMA.ConfGeneral.Modo === 'Visor') { // se comprueba el modo
    HABLA(0, 'Se inicio en  modo Visor', 'success');
    $('#signal').addClass('parpadea');
  } else {
    $.get("/iniciar", function(data, status){console.log('inicio');});    
    HABLA(0, 'Se inicio en modo Recolector', 'success');
    $('#signal').addClass('parpadea');
  }
  $('td#barralateral').css('background-color', '#11881161');
  $('#estado').html('En Linea');
  $('#mensaje').html('Encendido');
  $('#botoniniciar').attr('disabled', true);
  $('#botondetener').attr('disabled', false);
  RegistroActual();
  clearInterval(intervaloBarra);
  intervaloBarra = window.setInterval(function () { BarradeProgreso();}, 200);
}

async function iniciarAutomaticamente () {
  EstadoMenuInicio();  
  if (configGeneralEMA.ConfGeneral.inicio_automatico) {
    await esperar(4000);
    HABLA(0, 'Iniciando Automaticamente..', 'info');
    if ($('#mensaje').html() === 'Encienda EMA Libre') {
      await esperar(5000);
      INICIAR();
    } else {
      HABLA(0, 'No se pudo iniciar automaticamente, verifique el error', 'info');
    }
  }
}

function DETENER () {
    clearInterval(intervaloBarra);    
    $('td#barralateral').css('background-color', '#ff000030');
    $('#estado').html('Fuera de Linea');
    $('#img_estado').attr('src', iconoNeutro);
    $('#mensaje').html('EMA Libre se encuentra detenida');
    $('#mensajedetalle').html('Se detuvo manualmente.');
    $('#img_detalle').attr('src', iconoNeutro);
    $('#img_mensaje').attr('src', iconoNeutro);
    $('#botoniniciar').attr('disabled', false);
    $('#botondetener').attr('disabled', true);  
  if (configGeneralEMA.ConfGeneral.Modo === 'Visor') { // se comprueba el modo
    HABLA(0, 'Modo visor detenido', 'error');
    $('#signal').removeClass('parpadea');
  } else {
    $.get("/detener", function(data, status){console.log('detener');});  
    HABLA(0, 'Modo Recolector detenido', 'error');
    $('#signal').removeClass('parpadea');
  }
}

async function redimensionarInicio () 
  {	
  $('#loadinicio').attr('src', 'iconos/icono_app_guina.webp');	
  await esperar(500);
  $('#loadinicio').attr('src', 'iconos/icono_app.webp');	
  await esperar(600);
  $('#cargador').empty();
  $('#cargador').remove();
  $('#principal').show();
  //win.setMinimumSize(700, 500);
 // win.setMaximumSize(4000, 2000);
 // win.maximize();
  HABLA(0, '...', 'info');
  await esperar(1000);
  HABLA(0, 'Bienvenido a Ema Libre Cliente, me llaman Carpincho Web.', 'info');
  iniciarAutomaticamente();
  }



async function Actualizar () {
  var SO_WIN = plataforma.indexOf('win');
  if (SO_WIN === 0) {
    funcionesActualizar.Control(function (resultado, VersionActual, nueva) {
      $('#botonactualizar').attr('disabled', true);
      if (resultado && VersionActual) {
        DETENER();
        HABLA(0, 'Descargando nueva versión de Carpincho: ' + nueva, 'info');
        NProgress.set(0);
        $('#mensajebarra').html('Descargando nueva versión, espere ....');
        funcionesActualizar.Descargar(nueva, function (resultadoact, avance, porcentaje) {
         NProgress.set(porcentaje/100);
          $('#mensajebarra').html(avance);
          if (resultadoact && avance) {
            NProgress.set(1);
            $('#mensajebarra').html('Descomprimiendo la nueva versión, espere ....');
            HABLA(0, 'Descomprimiendo la nueva versión, espere ....', 'info');
            funcionesActualizar.Descomprimir(function (resultadoact) {
              if (resultadoact) {
                $('#mensajebarra').html('Borrado del archivo descargado, espere ....');
                funcionesActualizar.Desplegar(function (resultado) {
                  if (resultado) {
                    HABLA(0, 'Ejecutando App de Actualizacion, espere...', 'info');
                    setTimeout(funcionesActualizar.LanzarAplicativoIndependiente, 9000);
                  } else {
                    HABLA(0, 'Ocurrio un problema en el despliege, intente mas tarde', 'error');
                  }
                });
              } else {
                HABLA(0, 'Ocurrio un problema descomprimiendo, intente mas tarde', 'error');
              }
            });
          }
          if (!resultadoact && !avance) {
            HABLA(0, 'Ocurrio un problema en la descarga, intente mas tarde', 'error');
          }
        });
      }
      if (!resultado && VersionActual) {
        HABLA(0, 'No hay versiones nuevas disponibles', 'success');
      }
      if (!resultado && !VersionActual) {
        HABLA(0, 'No se pudo conectar al servidor de versiones,pruebe mas tarde...', 'error');
      }
    });
  } else {
    
    HABLA(0, 'Esta opción solo es válida para sistemas operativos Windows.', 'warn');
    await esperar(4000);
    HABLA(0, 'Pero usted puede actualizar EMA Libre con su sistema de paquetes nativo.', 'info');
    await esperar(4000);
    HABLA(0, 'O puede ingresar a https://ema.gugler.com.ar para más información. ', 'info');
  }
  $('#botonactualizar').attr('disabled', false);
}

async function HABLA (num, mesg, tipo) {
  var tamañomsg = mesg.length;
  if (num === 0) {
    $('#carpincho_img').notify(mesg, {
      className: tipo,
      position: 'right middle',
      autoHideDelay: (tamañomsg * 90)
    });
    if (configGeneralEMA.ConfGeneral.Audio) {
      window.speechSynthesis.cancel();
      var voices = window.speechSynthesis.getVoices();     
      var cantidadVoces = voices.length;
      //console.log(cantidadVoces);
      if( cantidadVoces > 0){
      var seleccionada = voices.filter(function (voice) { return voice.name === configGeneralEMA.ConfGeneral.audioVoces; })[0];
      //console.log(seleccionada);
      var msg = new SpeechSynthesisUtterance (mesg);
      msg.voice = seleccionada;
      msg.rate = 1.2;
      msg.pitch = 1;
      window.speechSynthesis.speak(msg);
      msg=null;seleccionada=null;voices=null;cantidadVoces=null;
      }
    }
  }
  if ($('#carpincho_img').attr('src') === 'iconos/icono_app.webp') {
    $('#carpincho_img').attr('src', 'iconos/icono_app_habla.webp');
  } else {
    $('#carpincho_img').attr('src', 'iconos/icono_app.webp');
  }
  num++;
  if (num < tamañomsg) {
    await esperar(90);
    HABLA(num ,mesg ,0);
  } else {
    $('#carpincho_img').attr('src', 'iconos/icono_app.webp');
    //LimpiarMemoria(true);
  }
  num=null;tamañomsg = null;
}

async function GUIÑA (num) 
      {
      if ($('#carpincho_img').attr('src') === 'iconos/icono_app.webp') {
            $('#carpincho_img').attr('src', 'iconos/icono_app_guina.webp');
          } else {
            $('#carpincho_img').attr('src', 'iconos/icono_app.webp');
          }
      num++;
      if (num < 2) {
        await esperar(400);
          GUIÑA(num);
          } else {
            $('#carpincho_img').attr('src', 'iconos/icono_app.webp');            
          }
      num =null;
      }


async function controlPendientes () {
      //var pendientes = EMA.HayPendientes();      
      $.get("/HayPendientes", function(data, status)
        { 
        var pendientes = data;
        var DIVPENDIENTE = document.getElementById('DIVPendientes');
        var SPANTANQUE = document.getElementById('Destanqueo');
        var DIVModo = document.getElementById('DIVModo');
        if (pendientes !== false) 
        {
          DIVPENDIENTE.style.visibility = 'visible';
          if ((pendientes < destanqueo)) {SPANTANQUE.style.visibility = 'visible'; } else {SPANTANQUE.style.visibility = 'hidden';  }
          destanqueo = pendientes;
          $('#Pendientes').html("<span class='label label-warning' title='Pendientes'> Cantidad: " + pendientes + '</span><br>');
        } else {
          destanqueo = 0;
          DIVPENDIENTE.style.visibility = 'hidden';
          SPANTANQUE.style.visibility = 'hidden';
          DIVModo.style.visibility = 'hidden';
        }
        pendientes = null;
        DIVPENDIENTE = null;
        SPANTANQUE = null;
        DIVModo = null;
        });
      return true;
    }

async function mostrarEstados (EstadoSensores) {
      // Se consulta por los sensores
      var Resultado = !((EstadoSensores.lost_sensor_contact) || (EstadoSensores.rain_overflow));
      if (Resultado && (configGeneralEMA.ConfGeneral.Modo !== 'Visor')) {
        $('#signal').attr('src', 'img/signal.webp');
        $('#signal').addClass('parpadea');
      } else {
        if (!EstadoSensores.lost_sensor_contact) {
          HABLA(0, 'Perdida de la señal de Radio Frecuencia', 'warn');
        }
        if (!EstadoSensores.rain_overflow) {
          HABLA(0, 'Sensor de lluvia externo sobrecargado', 'warn');
        }
        $('#signal').attr('src', 'img/no_signal.webp');
        $('#signal').addClass('parpadea');
       // win.requestAttention(3);     
      }
      EstadoSensores = null;
      Resultado = null;
    }

async function RegistroActual (barratermino) 
  {
  //console.log('ENTRO REGISTRO ACTUAL');
 
  if (configGeneralEMA.ConfGeneral.Modo === 'Recolector') 
    { // se comprueba el modo se encuentra en Recolector
      if(barratermino == true){
                                //BARRA TERMINO                
                                //console.log('ENTRO AL BARRA TERMINO RECOLECTOR');
                                await esperar(100);
                                EstadoMenuInicio();
                                await esperar(100);
                                MuestraRegistroActual(true);
                                await esperar(100);
                                Graficos();
                                await esperar(100);
                                ClimaActual();
                                await esperar(1500);
                                RegistroPublicado();
                                cant=0;
                                }

    $.get("/EMA_estado", async function(data, status){
    var EstadoEMA = data; 
    if (EstadoEMA[0] === true) 
      {
        $('#mensaje').html('Recolectando.');
        $('#img_mensaje').attr('src', iconoControlTrue);
        $('#mensajedetalle').html('Operando.');
        $('#img_detalle').attr('src', iconoControlTrue);
        controlPendientes();
        $.get("/EstadoSensores", function(data, status){
            var EstadoSensores = data;
            if(EstadoSensores !== false)
            { 
              mostrarEstados(EstadoSensores);
            }else{EstadoSensores=null;}
          });
        $.get("/UltimoRegistro", function(data, status){
                UltimoReg = data;     
                if (UltimoReg == false) { 
                    // Se consulta por la cantidad de errores
                      //await esperar(10000);
                      HABLA(0, 'No se pueden obtener registros. Desconecte y vuelva a conectar el USB. Se prueba reiniciar el servicio.', 'error');
                    } else {
                            //  $('#mensaje').html(' Sincronizando ...');
                             // $('#img_mensaje').attr('src', iconoNeutro);
                              //await esperar(200);
                            //  RegistroActual();
                              return false;
                            }
               });      
      } else {
               $('#mensaje').html('EmaLibre detenida.');
               $('#img_mensaje').attr('src', iconoControlFalse);
               $('#mensajedetalle').html(EstadoEMA[1]);
               $('#img_detalle').attr('src', iconoControlFalse);
               await esperar(5000);
               RegistroActual();
               return false;
             }  
             
    });
  } else { // Modo visor
          if(barratermino == true){
                                  //BARRA TERMINO                
                                  await esperar(100);
                                  EstadoMenuInicio();
                                  await esperar(100);
                                  MuestraRegistroActual(true);
                                  await esperar(100);
                                  Graficos();
                                  await esperar(100);
                                  ClimaActual();
                                  cant=0;
                                  }         
          $('#signal').attr('src', 'img/visor.webp');
          $.get("/Visor", function(data, status){
            if (data[0] != null) {
              HABLA(0, 'No se pueden obtener datos del servidor. ' + data[0], 'info');
              $('#signal').removeClass('parpadea');       
              setTimeout(function(){RegistroActual();}, 10000);
            } else {
              $('#signal').addClass('parpadea');
              var DIVModo = document.getElementById('DIVModo');
              DIVModo.style.visibility = 'visible';
              document.getElementById('DIVModo').innerHTML = "<span class='badge badge-primary' title='Modo Visor'> Modo Visor </span><br><span class='badge badge-primary' >" + data[1].pais + ', ' + data[1].ciudad + "</span><br><span class='badge badge-primary' >" + data[1].modelo + '</span>';
              UltimoReg = data[1].registros[0];
              ModeloVisor = data[1].nombre;
              // se convierte la fecha
              var db = new Date(data[1].registros[0].fecha);
              var minutos = db.getMinutes();
              if (minutos <= 9) { minutos = '0' + minutos; }
              var hora = db.getHours();
              if (hora <= 9) { hora = '0' + hora; }
              var mes = db.getMonth() + 1;
              if (mes <= 9) { mes = '0' + mes; }
              var dia = db.getDate();
              if (dia <= 9) { dia = '0' + dia; }
              var segundo = db.getSeconds();
              if (segundo <= 9) { segundo = '0' + segundo; }
              var fecha = dia + '/' + mes + '/' + db.getFullYear() + '  ' + hora + ':' + minutos + ':' + segundo;
              UltimoReg.fecha = fecha;
              fecha=null;dia=null;mes=null;segundo=null;minutos=null;hora=null;db=null;
              // ver acumulados de lluvia
              UltimoReg.lloviendo = data[1].registros[0].lluvia_actual;
              UltimoReg.lluvia_acu_hora = data[1].registros[0].lluvia_acumulado_hora;
              UltimoReg.lluvia_acu_diaria = data[1].registros[0].lluvia_acumulado_diario;
              UltimoReg.lluvia_acumulado_total = data[1].registros[0].lluvia_acumulado_total;
              // Maximas y minimas, no llegan de EMACENTER, por el momento se setean en 0.
              UltimoReg.temperatura_externa_max_dia = 0;
              UltimoReg.temperatura_externa_min_dia = 0;
              UltimoReg.temperatura_interna_max_dia = 0;
              UltimoReg.temperatura_interna_min_dia = 0;
              UltimoReg.punto_de_rocio_max_dia = 0;
              UltimoReg.punto_de_rocio_min_dia = 0;
              UltimoReg.humedad_externa_max_dia = 0;
              UltimoReg.humedad_externa_min_dia = 0;
              UltimoReg.humedad_interna_max_dia = 0;
              UltimoReg.humedad_interna_min_dia = 0;
              UltimoReg.luxer_intencidad_max_dia = 0;
              UltimoReg.luxer_intencidad_min_dia = 0;
              UltimoReg.luxer_uv_max_dia = 0;
              UltimoReg.luxer_uv_min_dia = 0;
              UltimoReg.presion_relativa_max_dia = 0;
              UltimoReg.presion_relativa_min_dia = 0;
              tiempovisor = tiempovisor - 0.2;
            }

          });  
        }
  EstadoEMA = null;    
}

async function BarradeProgreso () {
  if (configGeneralEMA.ConfGeneral.Modo === 'Recolector') 
  { // EN MODO RECOLECTOR    
    $.get("/TiempoActualRecolector", function(data, status){
      tiempoActualRecolector = data; 
          if(tiempoActualRecolector[0] !== false || tiempoActualRecolector[1] > 0)
          {

            if ((tiempoActualRecolector[0] < 98 && tiempoActualRecolector[1]  > 0.39) )
              {  
                // CONTINUA ADENTRO
                    NProgress.set(tiempoActualRecolector[0]/100);
                  return false;
              } else {                 
                    // Cuando termine cada barra, publica y luego vuelve al leer el registro actual                        
                    cant++;
                    if(cant == 1){ RegistroActual(true);} else{return true;};
                    // solo si esta en modo visor
                    if (configGeneralEMA.ConfGeneral.Modo !== 'Recolector') {document.getElementById('RedesMet').innerHTML = "<span class='badge badge-success' title='Obtenido Correctamente'>" + ModeloVisor + ': ' + UltimoReg.fecha.substring(10, 20) + '</span><br>';} 
                        NProgress.done();
              }
          }else{ if(cant > 0){return false;}
                    NProgress.done();
                }

    });       
  } else { // MODO VISOR                 
          if( tiempovisor != 99)
                {
                  if( tiempovisor >= configGeneralEMA.ConfGeneral.Tiempo_de_Recoleccion)
                  {
                    tiempovisor = 0;
                    RegistroActual(true);                    
                   NProgress.done();
                  }else{
                          tiempovisor = tiempovisor + 0.2; // se le suma el tiempo del intervalo
                          var tiempo_barra = ((tiempovisor * 100) / configGeneralEMA.ConfGeneral.Tiempo_de_Recoleccion)/100;
                          NProgress.set(tiempo_barra);
                        }
                }
         }  

  UltimoPub = null;aux=null;tiempoActualRecolector = null; tiempo_barra = null;
  
}


// **********************   Menu Acciones  *******************************************************************
function MenuConfig () {
  if ($('#Modo').val() === 'Recolector') {
    $('#LlaveR').attr('disabled', true);
    $('#LlaveR').val('');
    $('#LlaveR').hide();
  } else {
    $('#LlaveR').attr('disabled', false);
    $('#LlaveR').show();
    $('#Llave_R').val(configGeneralEMA.ConfGeneral.Llave_R);
  }
  if ($('#Tipo_SMTP').val() === 'Deshabilitado') {
    $('#Usuario_SMTP').attr('disabled', true);
    $('#Contrasenia_SMTP').attr('disabled', true);
    $('#Puerto_SMTP').attr('disabled', true);
    $('#Server_SMTP').attr('disabled', true);
    $('#Puerto_SMTP').val('');
    $('#puerto').hide();
    $('#Server_SMTP').val('');
    $('#Server').hide();
  } else {
    $('#Usuario_SMTP').attr('disabled', false);
    $('#Contrasenia_SMTP').attr('disabled', false);
    if ($('#Tipo_SMTP').val() === 'Manual') {
      $('#Puerto_SMTP').attr('disabled', false);
      $('#puerto').show();
      $('#Server_SMTP').attr('disabled', false);
      $('#Server').show();
    } else {
      $('#Puerto_SMTP').attr('disabled', true);
      $('#puerto').hide();
      $('#Server_SMTP').attr('disabled', true);
      $('#Server').hide();
    }
    $('#Usuario_SMTP').val(configGeneralEMA.ConfEmail.Usuario);
    $('#Contrasenia_SMTP').val(configGeneralEMA.ConfEmail.Password);
    $('#Server_SMTP').val(configGeneralEMA.ConfEmail.Server);
    $('#Puerto_SMTP').val(configGeneralEMA.ConfEmail.Puerto);
  }
  if ($('#audio').val() === 'Desactivado') {
    $('#vocesAudioDiv').attr('disabled', true);
    $('#vocesAudioDiv').val('');
    $('#vocesAudioDiv').hide();
  } else {
    $('#vocesAudioDiv').attr('disabled', false);
    $('#vocesAudioDiv').show();
    $('#vocesAudio').val(configGeneralEMA.ConfGeneral.audioVoces);
    $('#vocesAudio').selectpicker('render');
  }
}

async function MenuGUI (elemento) {
  try {
    document.getElementById("actualizando_inicio").hidden = false;
    document.getElementById("actualizando_inicio").style.display = "block";
    var seleccion;
    if (elemento === 'logs') {seleccion = document.getElementById('configuracion');}
     else {seleccion = document.getElementById(elemento);}
    $(seleccion).empty();
    $(seleccion).hide();
    $(seleccion).load('./view/modelo_' + elemento + '.html', function (datosArchivo) {
      switch (elemento) {
        case 'logs':
          VerRegistrosLog();
          $('#interior').empty();$('#exterior').empty();$('#inicio').empty();$('#contacto').empty();$('#inicio').empty();$('#estadisticas').empty();
          break;
        case 'redes':
          ListarRedes();
          $('#interior').empty();$('#exterior').empty();$('#contacto').empty();$('#configuracion').empty();$('#inicio').empty();$('#estadisticas').empty();
          break;
        case 'redes_alta':
          MenuSelectRed();
          break;
        case 'inicio':
          $('#node_pid').html(pid);
          $('#node_ppid').html(ppid);
          $('#nwjs_v').html(version_node);
          EstadoMenuInicio();
          $('#interior').empty();$('#exterior').empty();$('#contacto').empty();$('#configuracion').empty();$('#redes').empty();$('#estadisticas').empty();
          break;
        case 'interior':
          MuestraRegistroActual(false,'interior');
          Graficos('interior');
          $('#inicio').empty();$('#exterior').empty();$('#contacto').empty();$('#configuracion').empty();$('#redes').empty();$('#estadisticas').empty();          
          break;
        case 'exterior':
          MuestraRegistroActual(false,'exterior');
          Graficos('exterior');
          $('#interior').empty();$('#inicio').empty();$('#contacto').empty();$('#configuracion').empty();$('#redes').empty();$('#estadisticas').empty();
          break;
        case 'estadisticas':           
          graficoEstadisticas();
          $('#interior').empty();$('#exterior').empty();$('#inicio').empty();$('#contacto').empty();$('#configuracion').empty();$('#redes').empty();
          break;          
        case 'contacto':
          var NombreVersion = ' v' + configGlobalEMA.ConfGlobal.Version;
          $('#versioncontacto').html(NombreVersion);
          $('#interior').empty();$('#exterior').empty();$('#inicio').empty();$('#configuracion').empty();$('#redes').empty();$('#estadisticas').empty();
          break;
        case 'configuracion':
          $('#interior').empty();$('#exterior').empty();$('#contacto').empty();$('#inicio').empty();$('#redes').empty();$('#estadisticas').empty();
        // Se obtienen las voces del sistema
          var synth = window.speechSynthesis;
          const voice = synth.getVoices();
          var voiceSelect = document.getElementById('vocesAudio');         
          var cantidadVoces = voice.length;
          if( cantidadVoces > 0){
                                for (var i = 0; i < cantidadVoces; i++) 
                                  {
                                  var option = document.createElement('option');                        
                                  option.textContent = voice[i].name;
                                  option.setAttribute('value', voice[i].name);
                                  voiceSelect.appendChild(option);
                                  }
                                }else{
                                    var option = document.createElement('option');  
                                    voice[0] = 'Sin Voces';
                                    option.textContent = 'Sin voces, verifique el SO';
                                    option.setAttribute('value', 'Sin Voces');
                                    voiceSelect.appendChild(option);
                                    voiceSelect.selectedIndex = "0";
                                  }

          $('#NombreEstacion').val(configGeneralEMA.ConfGeneral.NombreEstacion);
          $('#Modo').val(configGeneralEMA.ConfGeneral.Modo);
          $('#Modo').selectpicker('render');
          $('#SistemaMetrico').val(configGeneralEMA.ConfGeneral.SistemaMetrico);
          $('#SistemaMetrico').selectpicker('render');
          $('#Llave_R').val(configGeneralEMA.ConfGeneral.Llave_R);
          $('#Tiempo_de_Recoleccion').val(configGeneralEMA.ConfGeneral.Tiempo_de_Recoleccion);
          $('#Email_receptor').val(configGeneralEMA.ConfEmail.ReceptorEmail);
          $('#Tipo_SMTP').val(configGeneralEMA.ConfEmail.Servicio);
          $('#Tipo_SMTP').selectpicker('render');
          $('#Tiempo_de_Recoleccion').selectpicker('render');
          $('#Debug').val(configGeneralEMA.ConfGeneral.debug);
          $('#debugMB').val(configGeneralEMA.ConfGeneral.debugTamanio);
          $('#Debug').selectpicker('render');
          var automatico;
          if (configGeneralEMA.ConfGeneral.inicio_automatico) { automatico = 'Activado'; } else { automatico = 'Desactivado'; }
          $('#inicio_auto').val(automatico);
          $('#inicio_auto').selectpicker('render');
          var cambios;
          if (configGeneralEMA.ConfGeneral.notificar_cambios) { cambios = 'Activado'; } else { cambios = 'Desactivado'; }
          $('#notificar_cambios').val(cambios);
          $('#notificar_cambios').selectpicker('render');          
          var Audio;
          if (configGeneralEMA.ConfGeneral.Audio) { Audio = 'Activado'; } else { Audio = 'Desactivado'; }
          $('#audio').val(Audio);
          $('#audio').selectpicker('render');
          $('#utc').val(configGeneralEMA.ConfGeneral.UTC);
          $('#utc').selectpicker('render');
          // Control para el audio
          if (configGeneralEMA.ConfGeneral.Audio === false) {
            $('#vocesAudioDiv').attr('disabled', true);
            $('#vocesAudioDiv').val('');
            $('#vocesAudioDiv').hide();
          } else {
            $('#vocesAudioDiv').attr('disabled', false);
            $('#vocesAudioDiv').show();
            if (configGeneralEMA.ConfGeneral.audioVoces === null || typeof configGeneralEMA.ConfGeneral.audioVoces === 'undefined') { $('#vocesAudio').val(voice[0].name); } else { $('#vocesAudio').val(configGeneralEMA.ConfGeneral.audioVoces); }
          }
          $('#vocesAudio').selectpicker('render');
          // Control para el modo
          if ($('#Modo').val() === 'Recolector') {
            $('#LlaveR').attr('disabled', true);
            $('#LlaveR').val('');
            $('#LlaveR').hide();
          } else {
            $('#LlaveR').attr('disabled', false);
            $('#LlaveR').show();
            $('#Llave_R').val(configGeneralEMA.ConfGeneral.Llave_R);
          }
          // Control para el SMTP
          if ($('#Tipo_SMTP').val() === 'Deshabilitado') {
            $('#Usuario_SMTP').attr('disabled', true);
            $('#Contrasenia_SMTP').attr('disabled', true);
            $('#Puerto_SMTP').attr('disabled', true);
            $('#Server_SMTP').attr('disabled', true);
            $('#Server').hide();
            $('#puerto').hide();
          } else {
            $('#Usuario_SMTP').attr('disabled', false);
            $('#Contrasenia_SMTP').attr('disabled', false);
            $('#Server_SMTP').attr('disabled', false);
            $('#Puerto_SMTP').attr('disabled', false);
            $('#Usuario_SMTP').val(configGeneralEMA.ConfEmail.Usuario);
            $('#Contrasenia_SMTP').val(configGeneralEMA.ConfEmail.Password);
            $('#Server_SMTP').val(configGeneralEMA.ConfEmail.Server);
            $('#Puerto_SMTP').val(configGeneralEMA.ConfEmail.Puerto);
            if ($('#Tipo_SMTP').val() === 'Manual') {
              $('#Server_SMTP').attr('disabled', false);
              $('#Puerto_SMTP').attr('disabled', false);
              $('#puerto').show();
              $('#Server').show();
            } else {
              $('#puerto').hide();
              $('#Server').hide();
            }
          }
          break;
        default:
          console.log('No existe el menu seleccionado: ' + elemento);
      }
      setTimeout(function(){ document.getElementById("actualizando_inicio").style.display = "none";
                             document.getElementById("actualizando_inicio").hidden = false;
                             $(seleccion).show();
                            }, 200);  

    });

  } catch (e) { // Si falla se informa y se vuelve a cargar el modelo interior
    HABLA(0, 'Se realizó un reload de la libreria grafica interior', 'info');
  }
}

async function autoLoad () 
  {
  await esperar(6000);
  ComprobarEstado();
  //var plataforma = process.platform;
  //var SO_WIN = plataforma.indexOf('win');
  //if (SO_WIN === 0) { funcionesActualizar.Control(function (resultado, VersionActual, nueva) { if (resultado) { HABLA(0, 'Hay una nueva version de Carpincho. Versión ' + nueva, 'error'); } }); }
  }
autoLoad();